@@ -1,7 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 #
-# Copyright (C) 2004-2005 Edgewall Software
+# Copyright (C) 2004-2006 Edgewall Software
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
+# Copyright (C) 2006 Christian Boos <cboos@neuf.fr>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -151,11 +152,9 @@ class RoadmapModule(Component):
         req.hdf['roadmap.showall'] = showall
 
         db = self.env.get_db_cnx()
-        milestones = []
-        for idx, milestone in enum(Milestone.select(self.env, showall)):
-            hdf = milestone_to_hdf(self.env, db, req, milestone)
-            milestones.append(hdf)
-        req.hdf['roadmap.milestones'] = milestones
+        milestones = [milestone_to_hdf(self.env, db, req, m)
+                      for m in Milestone.select(self.env, showall, db)]
+        req.hdf['roadmap.milestones'] = milestones        
 
         for idx,milestone in enum(milestones):
             milestone_name = unescape(milestone['name']) # Kludge
@@ -255,7 +254,7 @@ class RoadmapModule(Component):
                            if ticket['owner'] == user]:
                 ticket = Ticket(self.env, tkt_id)
                 write_prop('BEGIN', 'VTODO')
-                if milestone.has_key('date'):
+                if milestone.has_key('due'):
                     write_prop('RELATED-TO', uid)
                     write_date('DUE', localtime(milestone['due']))
                 write_prop('SUMMARY', 'Ticket #%i: %s' % (ticket.id,
@@ -318,13 +317,13 @@ class MilestoneModule(Component):
                 title = Markup('Milestone <em>%s</em> completed', name)
                 if format == 'rss':
                     href = self.env.abs_href.milestone(name)
-                    message = wiki_to_html(description or '--', self.env,
-                                           req, db, absurls=True)
+                    message = wiki_to_html(description, self.env, db,
+                                           absurls=True)
                 else:
                     href = self.env.href.milestone(name)
                     message = wiki_to_oneliner(description, self.env, db,
                                                shorten=True)
-                yield 'milestone', href, title, completed, None, message
+                yield 'milestone', href, title, completed, None, message or '--'
 
     # IRequestHandler methods
 
