@@ -1,7 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 #
-# Copyright (C) 2003-2005 Edgewall Software
+# Copyright (C) 2003-2006 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgstrm <jonas@edgewall.com>
+# Copyright (C) 2005-2006 Christian Boos <cboos@neuf.fr>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -19,6 +20,7 @@ import re
 import urllib
 
 from trac import util
+from trac.util import sorted
 from trac.core import *
 from trac.mimeview import get_mimetype, is_binary, detect_unicode, Mimeview
 from trac.perm import IPermissionRequestor
@@ -32,22 +34,6 @@ IMG_RE = re.compile(r"\.(gif|jpg|jpeg|png)(\?.*)?$", re.IGNORECASE)
 
 CHUNK_SIZE = 4096
 
-DIGITS = re.compile(r'[0-9]+')
-def _natural_order(x, y):
-    """Comparison function for natural order sorting based on
-    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/214202."""
-    nx = ny = 0
-    while True:
-        a = DIGITS.search(x, nx)
-        b = DIGITS.search(y, ny)
-        if None in (a, b):
-            return cmp(x[nx:], y[ny:])
-        r = (cmp(x[nx:a.start()], y[ny:b.start()]) or
-             cmp(int(x[a.start():a.end()]), int(y[b.start():b.end()])))
-        if r:
-            return r
-        nx, ny = a.end(), b.end()
-
 
 class BrowserModule(Component):
 
@@ -93,8 +79,7 @@ class BrowserModule(Component):
         node = get_existing_node(self.env, repos, path, rev)
 
         hidden_properties = [p.strip() for p
-                             in self.config.get('browser', 'hide_properties',
-                                                'svk:merge').split(',')]
+                             in self.config.get('browser', 'hide_properties').split(',')]
         req.hdf['title'] = path
         req.hdf['browser'] = {
             'path': path,
@@ -148,20 +133,22 @@ class BrowserModule(Component):
             })
         changes = get_changes(self.env, repos, [i['rev'] for i in info])
 
-        def cmp_func(a, b):
-            dir_cmp = (a['is_dir'] and -1 or 0) + (b['is_dir'] and 1 or 0)
-            if dir_cmp:
-                return dir_cmp
-            neg = desc and -1 or 1
-            if order == 'date':
-                return neg * cmp(changes[b['rev']]['date_seconds'],
-                                 changes[a['rev']]['date_seconds'])
-            elif order == 'size':
-                return neg * cmp(a['content_length'], b['content_length'])
-            else:
-                return neg * _natural_order(a['name'].lower(),
-                                            b['name'].lower())
-        info.sort(cmp_func)
+        if order == 'date':
+            def file_order(a):
+                return changes[a['rev']]['date_seconds']
+        elif order == 'size':
+            def file_order(a):
+                return (a['content_length'],
+                        util.embedded_numbers(a['name'].lower()))
+        else:
+            def file_order(a):
+                return util.embedded_numbers(a['name'].lower())
+
+        dir_order = desc and 1 or -1
+
+        def browse_order(a):
+            return a['is_dir'] and dir_order or 0, file_order(a)
+        info = sorted(info, key=browse_order, reverse=desc)
 
         req.hdf['browser.items'] = info
         req.hdf['browser.changes'] = changes
